@@ -1,6 +1,6 @@
 /* gcompris - clickgame.c
  *
- * Time-stamp: <2001/12/10 00:01:07 bruno>
+ * Time-stamp: <2002/04/14 00:20:48 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -28,6 +28,8 @@
 
 #define SOUNDLISTFILE PACKAGE
 
+gboolean board_paused = TRUE;
+
 static GList *item_list = NULL;
 static GList *item2del_list = NULL;
 
@@ -52,7 +54,7 @@ static void	 clickgame_pause (gboolean pause);
 static void	 clickgame_end (void);
 static gboolean	 clickgame_is_our_board (GcomprisBoard *gcomprisBoard);
 static void	 clickgame_set_level (guint level);
-static void	 clickgame_help(void);
+static void	 clickgame_config(void);
 
 static FishItem *clickgame_create_item(GnomeCanvasGroup *parent);
 static gint	 clickgame_drop_items (GtkWidget *widget, gpointer data);
@@ -62,9 +64,12 @@ static void	 clickgame_destroy_item(FishItem *fishitem);
 static void	 clickgame_destroy_items(void);
 static void	 clickgame_destroy_all_items(void);
 static void	 setup_item(FishItem *fishitem);
-static void	 load_random_pixmap();
+static void	 load_random_pixmap(void);
 static void	 clickgame_next_level(void);
 
+static int gamewon;
+static void	 game_won(void);
+
 static  guint32              fallSpeed = 0;
 static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
@@ -88,7 +93,7 @@ BoardPlugin clickgame_bp =
    NULL,
    NULL,
    clickgame_set_level,
-   clickgame_help,
+   clickgame_config,
    NULL
 };
 
@@ -114,6 +119,11 @@ static void clickgame_pause (gboolean pause)
   if(gcomprisBoard==NULL)
     return;
 
+  if(gamewon == TRUE && pause == FALSE) /* the game is won */
+    {
+      game_won();
+    }
+
   if(pause)
     {
       if (dummy_id) {
@@ -142,6 +152,8 @@ static void clickgame_pause (gboolean pause)
 	animate_id = gtk_timeout_add (200, (GtkFunction) clickgame_animate_items, NULL);
       }
     }
+
+  board_paused = pause;
 }
 
 /*
@@ -188,7 +200,7 @@ static void clickgame_start (GcomprisBoard *agcomprisBoard)
 			   gcomprisBoard->width - 220, 
 			   gcomprisBoard->height - 50, 
 			   gcomprisBoard->number_of_sublevel);
-      gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_HELP);
+      gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       clickgame_next_level();
 
@@ -209,6 +221,7 @@ clickgame_end ()
       clickgame_destroy_all_items();
       gcomprisBoard->level = 1;       // Restart this game to zero
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -240,31 +253,11 @@ clickgame_is_our_board (GcomprisBoard *gcomprisBoard)
 }
 
 static void
-clickgame_help ()
+clickgame_config ()
 {
   if(gcomprisBoard!=NULL)
     {
       clickgame_pause(TRUE);
-      gcompris_help_start(N_("Practice the computer mouse"), 
-			  N_("\
-Goal:\n\
-In this game, the children will discover the computer mouse\n\
-The goal is just to click with the left mouse button on the moving fish.\n\
-\n\
-Credits:\n\
-Fish are taken from the Unix utility xfishtank.\n\
-Images are taken from the National Undersearch Research Program (NURP) Collection\n\
-at http://www.photolib.noaa.gov\n\
-All images credit bellongs to the OAR/National Undersea Research Program (NURP)\n\
-Except for the first image that also credits the University of North Carolina at Wilmington\n\
-In the gcompris level order, images information is:\n\
-Location: Tropical Atlantic Ocean, Florida Keys (image nur00523)\n\
-Photographer: D. Kesling\n\
-Location: Florida Keys (image nur03006)\n\
-Location: Unknown (image nur03505)\n\
-Location: Unknown (image nur03010)\n\
-Location: Unknown (image nur03011)\n\
-Location: Unknown (image nur03013)\n"));
     }
 }
 
@@ -537,6 +530,24 @@ static gint clickgame_drop_items (GtkWidget *widget, gpointer data)
   return (FALSE);
 }
 
+/* ==================================== */
+static void game_won()
+{
+  gcomprisBoard->sublevel++;
+
+  if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
+    /* Try the next level */
+    gcomprisBoard->sublevel=1;
+    gcomprisBoard->level++;
+    if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+      board_finished(BOARD_FINISHED_RANDOM);
+      return;
+    }
+    gcompris_play_sound (SOUNDLISTFILE, "bonus");
+  }
+  clickgame_next_level();
+}
+
 static gint
 item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
 {
@@ -546,7 +557,10 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
    static int dragging;
    double item_x, item_y;
 
-   if(!get_board_playing())
+   if(!gcomprisBoard)
+     return FALSE;
+
+   if(board_paused)
      return FALSE;
 
    item_x = event->button.x;
@@ -577,19 +591,14 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, FishItem *fishitem)
              {
 	       clickgame_destroy_item(fishitem);
 	       gcompris_play_sound (SOUNDLISTFILE, "gobble");
-
+	       
 	       gcomprisBoard->sublevel++;
 	       gcompris_score_set(gcomprisBoard->sublevel);
-
-	       if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) {
-		 /* Try the next level */
-		 gcomprisBoard->level++;
-		 if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
-		   board_finished(BOARD_FINISHED_RANDOM);
-		   return FALSE;
-		 }
-		 clickgame_next_level();
-		 gcompris_play_sound (SOUNDLISTFILE, "bonus");
+	       
+	       if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
+		 gamewon = TRUE;
+		 clickgame_destroy_all_items();
+		 gcompris_display_bonus(gamewon, BONUS_FLOWER);
 		 return FALSE;
 	       }
 	       /* Drop a new item now to speed up the game */
