@@ -1,6 +1,6 @@
 /* gcompris - clockgame.c
  *
- * Time-stamp: <2001/12/02 22:43:17 bruno>
+ * Time-stamp: <2002/04/16 23:56:15 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -49,25 +49,25 @@ typedef struct {
 } Time;
 static Time timeToFind, currentTime;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-static void process_ok(void);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
+static void		 process_ok(void);
 
 static int gamewon;
-static void game_won();
+static void		 game_won(void);
 
-static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent);
-static void clockgame_destroy_item(GnomeCanvasItem *item);
-static void clockgame_destroy_all_items(void);
-static void get_random_hour(Time *time);
-static void clockgame_next_level(void);
-static gint item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-static void display_hour(guint hour);
-static void display_minute(guint minute);
-static void display_second(guint second);
+static GnomeCanvasItem	*clockgame_create_item(GnomeCanvasGroup *parent);
+static void		 clockgame_destroy_item(GnomeCanvasItem *item);
+static void		 clockgame_destroy_all_items(void);
+static void		 get_random_hour(Time *time);
+static void		 clockgame_next_level(void);
+static gint		 item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
+static void		 display_hour(guint hour);
+static void		 display_minute(guint minute);
+static void		 display_second(guint second);
 
 /* Description of this plugin */
 BoardPlugin menu_bp =
@@ -132,12 +132,12 @@ static void start_board (GcomprisBoard *agcomprisBoard)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-bg.jpg");
+      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "clockgame/clockgame-bg.jpg");
 
 
-      /* set initial values for this level */
+      /* set initial values for this level adjusted to fit the watch background */
       cx =  gcomprisBoard->width/2;
-      cy =  gcomprisBoard->height*0.4;
+      cy =  gcomprisBoard->height*0.4 + 42;
       clock_size = gcomprisBoard->height*0.3;
 
       gcomprisBoard->level=1;
@@ -154,6 +154,8 @@ static void start_board (GcomprisBoard *agcomprisBoard)
 
       gamewon = FALSE;
       pause_board(FALSE);
+
+      gcompris_set_cursor(GCOMPRIS_LINE_CURSOR);
     }
 
 }
@@ -168,6 +170,8 @@ end_board ()
       gcompris_score_end();
       clockgame_destroy_all_items();
     }
+  gcomprisBoard = NULL;
+  gcompris_set_cursor(GCOMPRIS_DEFAULT_CURSOR);
 }
 
 static void
@@ -315,7 +319,7 @@ static void display_minute(guint minute)
   canvasPoints->coords[3]=cy - needle_size * cos(ang);
   gnome_canvas_item_set (minute_item,
 			 "points", canvasPoints,
-			 "fill_color", "green",
+			 "fill_color", "red",
 			 "width_units", (double)1,
 			 "width_pixels", (uint) 4,
 			 "last_arrowhead", TRUE,
@@ -347,7 +351,7 @@ static void display_second(guint second)
   canvasPoints->coords[3]=cy - needle_size * cos(ang);
   gnome_canvas_item_set (second_item,
 			 "points", canvasPoints,
-			 "fill_color", "red",
+			 "fill_color_rgba", 0x68c46f00,
 			 "width_units", (double)1,
 			 "width_pixels", (uint) 4,
 			 "last_arrowhead", TRUE,
@@ -376,11 +380,11 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
   gchar *mtext = NULL;
 
   /* Load a gdk font */
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-140-*-*-*-*-*-*");
-  gdk_font_small = gdk_font_load ("-adobe-times-medium-r-normal--*-120-*-*-*-*-*-*");
-  gdk_font_big = gdk_font_load ("-adobe-times-medium-r-normal--*-140-*-*-*-*-*-*");
-  gdk_font_big2 = gdk_font_load ("-adobe-times-medium-r-normal--*-180-*-*-*-*-*-*");
-  gdk_font_big3 = gdk_font_load ("-adobe-times-bold-r-normal--*-240-*-*-*-*-*-*");
+  gdk_font = gdk_font_load (FONT_BOARD_SMALL);
+  gdk_font_small = gdk_font_load (FONT_BOARD_TINY);
+  gdk_font_big = gdk_font_load (FONT_BOARD_SMALL);
+  gdk_font_big2 = gdk_font_load (FONT_BOARD_MEDIUM);
+  gdk_font_big3 = gdk_font_load (FONT_BOARD_BIG_BOLD);
 
   canvasPoints = gnome_canvas_points_new (2);
 
@@ -393,7 +397,7 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 	  canvasPoints->coords[0]=cx + (needle_size-hour_point_size) * sin(ang);
 	  canvasPoints->coords[1]=cy - (needle_size-hour_point_size) * cos(ang);
 	  color="darkblue";
-	  color_text="green";
+	  color_text="red";
 	  font = gdk_font;
 	}
       else
@@ -401,8 +405,8 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 	  /* Minute point */
 	  canvasPoints->coords[0]=cx + (needle_size-min_point_size) * sin(ang);
 	  canvasPoints->coords[1]=cy - (needle_size-min_point_size) * cos(ang);
-	  color="green";
-	  color_text="lightgreen";
+	  color="red";
+	  color_text="red";
 	  font = gdk_font_small;
 	}
 
@@ -530,12 +534,12 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
   item = 
     gnome_canvas_item_new (parent,
 			   gnome_canvas_text_get_type (),
-			   "text", _("Set the clock to:"),
+			   "text", _("Set the watch to:"),
 			   "font_gdk", gdk_font_big3,
-			   "x", (double) cx,
-			   "y", (double) cy + needle_size +  needle_size / 3,
+			   "x", (double) gcomprisBoard->width*0.15,
+			   "y", (double) cy + needle_size +  needle_size / 3 - 30,
 			   "anchor", GTK_ANCHOR_CENTER,
-			   "fill_color", "green",
+			   "fill_color", "white",
 			   NULL);
   item_list = g_list_append (item_list, item);
   
@@ -544,10 +548,10 @@ static GnomeCanvasItem *clockgame_create_item(GnomeCanvasGroup *parent)
 			   gnome_canvas_text_get_type (),
 			   "text", "",
 			   "font_gdk", gdk_font_big3,
-			   "x", (double) cx,
-			   "y", (double) cy + needle_size +  needle_size / 3 + 30,
+			   "x", (double) gcomprisBoard->width*0.15,
+			   "y", (double) cy + needle_size +  needle_size / 3,
 			   "anchor", GTK_ANCHOR_CENTER,
-			   "fill_color", "green",
+			   "fill_color", "white",
 			   NULL);
   display_digital_time(time_to_find_item, &timeToFind);
   item_list = g_list_append (item_list, time_to_find_item);
