@@ -1,6 +1,6 @@
 /* gcompris - gletters.c
  *
- * Time-stamp: <2001/12/03 00:23:30 bruno>
+ * Time-stamp: <2002/02/03 09:00:52 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  * 
@@ -109,7 +109,6 @@ BoardPlugin
  */
 static void pause_board (gboolean pause)
 {
-
   if(gcomprisBoard==NULL)
     return;
 
@@ -169,6 +168,7 @@ end_board ()
       gcompris_score_end();
       gletters_destroy_all_items();
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -188,8 +188,8 @@ gint key_press(guint keyval)
   gchar *old_name;
   guint c;
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -424,6 +424,9 @@ static GnomeCanvasItem *gletters_create_item(GnomeCanvasGroup *parent)
 
   } while(item_find_by_title(lettersItem)!=NULL);
 
+  lettersItem[1] = '\0';
+  gcompris_play_ogg(lettersItem, NULL);
+
   str = g_strdup_printf("gcompris/letters/%c.png",
 			letters_array[gcomprisBoard->level%LETTERS_ARRAY_LENGTH][i]);
 
@@ -477,13 +480,14 @@ static void player_win(GnomeCanvasItem *item)
   gcomprisBoard->sublevel++;
   gcompris_score_set(gcomprisBoard->sublevel);
 
-  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) 
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel)
     {
       /* Try the next level */
       gcomprisBoard->level++;
       if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
-	board_finished(BOARD_FINISHED_RANDOM);
-	return;
+        gcompris_score_end();
+				board_finished(BOARD_FINISHED_RANDOM);
+				return;
       }
       gletters_next_level();
       gcompris_play_sound (SOUNDLISTFILE, "bonus");
