@@ -32,17 +32,17 @@
 GcomprisBoard *gcomprisBoard = NULL;
 gboolean board_paused = TRUE;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
 static int gamewon;
 static gint process_time_id = 0;
-static void process_ok(void);
-static void process_time(void);
-static void highlight_selected(GnomeCanvasItem *);
-static void game_won();
+static void		 process_ok(void);
+static void		 process_time(void);
+static void		 highlight_selected(GnomeCanvasItem *);
+static void		 game_won(void);
 
 typedef struct _Board Board;
 struct _Board {
@@ -57,13 +57,13 @@ struct _Board {
 static Board * board;
 
 /* XML */
-static gboolean read_xml_file(char *fname);
-static void init_xml();
-static void add_xml_data(xmlDocPtr, xmlNodePtr, GNode *);
-static void parse_doc(xmlDocPtr doc);
-static gboolean read_xml_file(char *fname);
-static void destroy_board_list();
-static void destroy_board(Board * board);
+static gboolean		 read_xml_file(char *fname);
+static void		 init_xml(void);
+static void		 add_xml_data(xmlDocPtr, xmlNodePtr, GNode *);
+static void		 parse_doc(xmlDocPtr doc);
+static gboolean		 read_xml_file(char *fname);
+static void		 destroy_board_list();
+static void		 destroy_board(Board * board);
 
 /* This is the list of boards */
 static GList *board_list = NULL;
@@ -186,6 +186,7 @@ end_board ()
       missing_letter_destroy_all_items();
       destroy_board_list();
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -262,7 +263,7 @@ static GnomeCanvasItem *missing_letter_create_item(GnomeCanvasGroup *parent)
   gint txt_area_x = 515;
   gint txt_area_y = 430;
 
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-240-*-*-*-*-*-*");
+  gdk_font = gdk_font_load (FONT_BOARD_BIG);
 
   board_number = (gcomprisBoard->level-1) *2 + gcomprisBoard->sublevel-1;
 /*  if (board_number >= g_list_length(board_list))
@@ -335,7 +336,7 @@ static GnomeCanvasItem *missing_letter_create_item(GnomeCanvasGroup *parent)
     buf[2] = board->l3;
     break;
   }
-  
+
   yOffset = ( gcomprisBoard->height - 3*gdk_pixbuf_get_height(button_pixmap) - 2*VERTICAL_SEPARATION) / 2;
   xOffset = (img_area_x-gdk_pixbuf_get_width(button_pixmap))/2;
   button1 = gnome_canvas_item_new (boardRootItem,
@@ -346,7 +347,7 @@ static GnomeCanvasItem *missing_letter_create_item(GnomeCanvasGroup *parent)
 				   NULL);
   l1_item = gnome_canvas_item_new (boardRootItem,
 				   gnome_canvas_text_get_type (),
-				   "text", _(buf[0]),
+				   "text", buf[0],
 				   "font_gdk", gdk_font,
 				   "x", (double) xOffset + gdk_pixbuf_get_width(button_pixmap)/2,
 				   "y", (double) yOffset + gdk_pixbuf_get_height(button_pixmap)/2,
@@ -363,7 +364,7 @@ static GnomeCanvasItem *missing_letter_create_item(GnomeCanvasGroup *parent)
 				   NULL);
   l2_item = gnome_canvas_item_new (boardRootItem,
 				   gnome_canvas_text_get_type (),
-				   "text", _(buf[1]),
+				   "text", buf[1],
 				   "font_gdk", gdk_font,
 				   "x", (double) xOffset + gdk_pixbuf_get_width(button_pixmap)/2,
 				   "y", (double) yOffset + gdk_pixbuf_get_height(button_pixmap)/2,
@@ -381,7 +382,7 @@ static GnomeCanvasItem *missing_letter_create_item(GnomeCanvasGroup *parent)
 				   NULL);
   l3_item = gnome_canvas_item_new (boardRootItem,
 				   gnome_canvas_text_get_type (),
-				   "text", _(buf[2]),
+				   "text", buf[2],
 				   "font_gdk", gdk_font,
 				   "x", (double) xOffset + gdk_pixbuf_get_width(button_pixmap)/2,
 				   "y", (double) yOffset + gdk_pixbuf_get_height(button_pixmap)/2,
@@ -422,7 +423,8 @@ static void process_ok()
   if (gamewon) {
     gnome_canvas_item_set(text, "text", board->answer, NULL);
   }
-  process_time_id = gtk_timeout_add (2000, (GtkFunction) process_time, NULL);
+  if(!process_time_id)
+    process_time_id = gtk_timeout_add (2000, (GtkFunction) process_time, NULL);
 }
 /* ==================================== */
 static void process_time()
@@ -516,9 +518,10 @@ static void init_xml()
 /* ==================================== */
 static void add_xml_data(xmlDocPtr doc, xmlNodePtr xmlnode, GNode * child)
 {
+	//int i;
   char *pixmapfile = NULL;
-  char question[255], answer[255];
-  char l1[2], l2[2], l3[2];
+  char *question = NULL, *answer = NULL;//question[255], answer[255];
+  char *l1 = NULL, *l2 = NULL, *l3 = NULL;//l1[2], l2[2], l3[2];
   char *data = NULL;
   Board * board = g_new(Board,1);
 
@@ -535,8 +538,10 @@ static void add_xml_data(xmlDocPtr doc, xmlNodePtr xmlnode, GNode * child)
     if (!strcmp(xmlnode->name, "data") && (lang==NULL
 					   || !strcmp(lang, gcompris_get_locale())
 					   || !strncmp(lang, gcompris_get_locale(), 2)))
-      data = xmlNodeListGetString(doc, xmlnode->xmlChildrenNode, 1);
-      data = convertUTF8Toisolat1(data);
+      {
+	data = xmlNodeListGetString(doc, xmlnode->xmlChildrenNode, 1);
+	data = convertUTF8Toisolat1(data);
+      }
     xmlnode = xmlnode->next;
   }
 
@@ -545,7 +550,16 @@ static void add_xml_data(xmlDocPtr doc, xmlNodePtr xmlnode, GNode * child)
   if ( pixmapfile == NULL || data == NULL)
     return;
 
-  sscanf(data, "%s %s %s %s %s", answer, question, l1, l2, l3);
+/*  if ((i=sscanf(data, "%s / %s / %s / %s / %s", answer, question, l1, l2, l3)) != 5)
+		printf("Error sscanf result != 5 = %i\n",i);
+*/
+	answer = strtok(data,"/");
+	question = strtok(NULL,"/");
+	l1 = strtok(NULL,"/");
+	l2 = strtok(NULL,"/");
+	l3 = strtok(NULL,"/");
+
+	assert(l1 != NULL  && l2 != NULL && l3 != NULL && answer != NULL && question != NULL);
 
   board->pixmapfile = g_strdup(pixmapfile);
   board->question = g_strdup(question);
