@@ -1,6 +1,6 @@
 /* gcompris - shapegame.c
  *
- * Time-stamp: <2001/12/09 23:45:54 bruno>
+ * Time-stamp: <2002/03/20 01:05:55 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -31,7 +31,7 @@
 
 #define UNDEFINED "Undefined"
 
-static GList *item_list = NULL;
+static int gamewon;
 
 static GcomprisBoard *gcomprisBoard = NULL;
 
@@ -47,25 +47,26 @@ typedef enum
 /* Let's define the structure for a single shape */
 typedef struct _Shape Shape;
 struct _Shape {
-  char  *name;			/* name of the shape */
-  char  *pixmapfile;		/* relative pixmap file name of the shape */
-  GnomeCanvasPoints* points;    /* OR list of points for this shape */
-  char  *targetfile;		/* OPTIONAL relative pixmap file name of the target shape, by default 
-				   a red point is displayed */
-  double x;			/* x coordinate */
-  double y;			/* y coordinate */
-  double w;			/* width */
-  double h;			/* height */
-  double zoomx;			/* x zoom factor */
-  double zoomy;			/* y zoom factor */
-  gint   position;		/* depth position 0=bottom other=intermediate */
-  ShapeType type;		/* Type of shape */
+  char  *name;				/* name of the shape */
+  char  *pixmapfile;			/* relative pixmap file name of the shape */
+  GnomeCanvasPoints* points; 		/* OR list of points for this shape */
+  char  *targetfile;			/* OPTIONAL relative pixmap file name of the target shape, by default 
+					   a red point is displayed */
+  double x;				/* x coordinate */
+  double y;				/* y coordinate */
+  double w;				/* width */
+  double h;				/* height */
+  double zoomx;				/* x zoom factor */
+  double zoomy;				/* y zoom factor */
+  gint   position;			/* depth position 0=bottom other=intermediate */
+  ShapeType type;			/* Type of shape */
 
-  GnomeCanvasItem *item;       	/* Canvas item for this shape */
-  Shape *icon_shape;		/* Temporary Canvas icon shape for this shape */
-  GnomeCanvasItem *bad_item;	/* Temporary bad placed Canvas item for this shape */
+  GnomeCanvasItem *item;     	  	/* Canvas item for this shape */
+  Shape *icon_shape;			/* Temporary Canvas icon shape for this shape */
+  GnomeCanvasItem *bad_item;		/* Temporary bad placed Canvas item for this shape */
 
-  gboolean found;		/* The user found this item */
+  gboolean found;			/* The user found this item */
+  GnomeCanvasItem *target_point;       	/* Target point item for this shape */
 };
 
 gchar *colorlist [] = 
@@ -89,27 +90,28 @@ static GList *shape_list = NULL;
 static GHashTable *shapelist_table = NULL;
 static gint SHAPE_BOX_WIDTH_RATIO = 18;
 
-static GnomeCanvasItem *shape_root_item;
-static GnomeCanvasItem *shape_list_root_item;
+static GnomeCanvasItem	*shape_root_item;
+static GnomeCanvasItem	*shape_list_root_item;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-static void process_ok(void);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void 		 pause_board (gboolean pause);
+static void 		 end_board (void);
+static gboolean 	 is_our_board (GcomprisBoard *gcomprisBoard);
+static void 		 set_level (guint level);
+static void 		 process_ok(void);
 
-static GnomeCanvasItem *shapegame_init_canvas(GnomeCanvasGroup *parent);
-static void shapegame_destroy_all_items(void);
-static void setup_item(GnomeCanvasItem *item, Shape *shape);
-static void shapegame_next_level(void);
-static gboolean read_xml_file(char *fname);
-static gboolean write_xml_file(char *fname);
-static Shape *find_closest_shape(double x, double y, double limit);
-static Shape *create_shape(ShapeType type, char *name, char *pixmapfile,  GnomeCanvasPoints* points,
-			   char *targetfile, double x, double y, double l, double h, double zoomx, 
-			   double zoomy, guint position);
-static void display_color_selector(GnomeCanvasGroup *parent);
+static GnomeCanvasItem 	*shapegame_init_canvas(GnomeCanvasGroup *parent);
+static void 		 shapegame_destroy_all_items(void);
+static void 		 setup_item(GnomeCanvasItem *item, Shape *shape);
+static void 		 shapegame_next_level(void);
+static gboolean 	 read_xml_file(char *fname);
+static gboolean 	 write_xml_file(char *fname);
+static Shape 		*find_closest_shape(double x, double y, double limit);
+static Shape 		*create_shape(ShapeType type, char *name, char *pixmapfile,  GnomeCanvasPoints* points,
+				      char *targetfile, double x, double y, double l, double h, double zoomx, 
+				      double zoomy, guint position);
+static gboolean 	 increment_sublevel(void);
+static void 		 create_title(char *name, double x, double y, char *justification);
 
 /* Description of this plugin */
 BoardPlugin menu_bp =
@@ -156,31 +158,78 @@ static void pause_board (gboolean pause)
   if(gcomprisBoard==NULL)
     return;
 
+  if(gamewon == TRUE && pause == FALSE) /* the game is won */
+    {
+      increment_sublevel();
+      shapegame_next_level();
+    }
+
 }
 
 /*
  */
 static void start_board (GcomprisBoard *agcomprisBoard)
 {
+  gchar *filename = NULL;
+  gboolean default_background = TRUE;
 
   if(agcomprisBoard!=NULL)
     {
       gcomprisBoard=agcomprisBoard;
 
-      gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-shapebg.jpg");
-
       /* set initial values for this level */
       gcomprisBoard->level = 1;
-      gcomprisBoard->maxlevel=6;
+
+      /* Calculate the maxlevel based on the available data file for this board */
+      gcomprisBoard->maxlevel=1;
+      filename = g_strdup_printf("%s/%s/board%d_0.xml",
+				 PACKAGE_DATA_DIR, gcomprisBoard->boarddir,
+				 gcomprisBoard->maxlevel);
+      while(g_file_exists(filename))
+	{
+	  gcomprisBoard->maxlevel++;
+	
+	  filename = g_strdup_printf("%s/%s/board%d_0.xml",
+				     PACKAGE_DATA_DIR, gcomprisBoard->boarddir,
+				     gcomprisBoard->maxlevel);
+	}
+      gcomprisBoard->maxlevel--;
+
+      g_free(filename);
+      
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_OK);
 
-      gcomprisBoard->number_of_sublevel=6;
-      gcomprisBoard->sublevel = 1;
+      gcomprisBoard->sublevel = 0;
+
+      /* In this board, the sublevels are dynamicaly discovered based on data files */
+      gcomprisBoard->number_of_sublevel=G_MAXINT;
+
+      
+      if(gcomprisBoard->mode!=NULL)
+	if(g_strncasecmp(gcomprisBoard->mode, "background=", 11)==0)
+	  {
+	    gchar *tmp = NULL;
+	    
+	    tmp = g_malloc(strlen(gcomprisBoard->mode));
+	    tmp = strcpy(tmp, gcomprisBoard->mode + 11);
+	    
+	    gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), tmp);
+	    default_background = FALSE;
+	    g_free(tmp);
+	  }
+
+      if(default_background)
+	{
+	  // Default case, load the default background
+	  gcompris_set_background(gnome_canvas_root(gcomprisBoard->canvas), "gcompris/gcompris-shapebg.jpg");
+	}
 
       shapegame_next_level();
 
       pause_board(FALSE);
 
+      gcompris_set_cursor(GCOMPRIS_LINE_CURSOR);
+
     }
 
 }
@@ -195,6 +244,8 @@ end_board ()
       shapegame_destroy_all_items();
       gcomprisBoard->level = 1;       // Restart this game to zero
     }
+  gcomprisBoard = NULL;
+  gcompris_set_cursor(GCOMPRIS_DEFAULT_CURSOR);
 }
 
 static void
@@ -204,7 +255,7 @@ set_level (guint level)
   if(gcomprisBoard!=NULL)
     {
       gcomprisBoard->level=level;
-      gcomprisBoard->sublevel=1;
+      gcomprisBoard->sublevel=0;
       shapegame_next_level();
     }
 }
@@ -238,15 +289,15 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 static gboolean increment_sublevel()
 {
   gcomprisBoard->sublevel++;
-  
-  if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
+
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) {
     /* Try the next level */
     gcomprisBoard->level++;
     if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
       board_finished(BOARD_FINISHED_RANDOM);
       return FALSE;
     }
-    
+
     gcomprisBoard->sublevel=0;
   }
 
@@ -255,7 +306,7 @@ static gboolean increment_sublevel()
 }
 
 /* set initial values for the next level */
-static void shapegame_next_level() 
+static void shapegame_next_level()
 {
   char *filename;
 
@@ -265,10 +316,13 @@ static void shapegame_next_level()
 
   shapegame_init_canvas(gnome_canvas_root(gcomprisBoard->canvas));
 
-  filename = g_strdup_printf("%s/%s/board%d_%d.xml",  
-			     PACKAGE_DATA_DIR, gcomprisBoard->boarddir, 
+  //  gcompris_set_background(GNOME_CANVAS_GROUP(shape_background_item), "gcompris/gcompris-shapebg.jpg");
+
+  filename = g_strdup_printf("%s/%s/board%d_%d.xml",
+			     PACKAGE_DATA_DIR, gcomprisBoard->boarddir,
 			     gcomprisBoard->level, gcomprisBoard->sublevel);
 
+
   while(!g_file_exists(filename)
 	&& ((gcomprisBoard->level != 1) || (gcomprisBoard->sublevel!=0)))
     {
@@ -276,14 +330,14 @@ static void shapegame_next_level()
       gcomprisBoard->sublevel=gcomprisBoard->number_of_sublevel;
       if(!increment_sublevel())
 	return;
-      
+
       g_free(filename);
-      filename = g_strdup_printf("%s/%s/board%d_%d.xml",  
-				 PACKAGE_DATA_DIR, gcomprisBoard->boarddir, 
+      filename = g_strdup_printf("%s/%s/board%d_%d.xml",
+				 PACKAGE_DATA_DIR, gcomprisBoard->boarddir,
 				 gcomprisBoard->level, gcomprisBoard->sublevel);
     }
   read_xml_file(filename);
-  
+
   g_free(filename);
 }
 
@@ -306,10 +360,8 @@ static void process_ok()
 
   if(done)
     {
-      increment_sublevel();
-
-      gcompris_play_sound (SOUNDLISTFILE, "bonus");
-      shapegame_next_level();
+      gamewon = TRUE;
+      gcompris_display_bonus(gamewon, BONUS_FLOWER);
     }
   else
     gcompris_play_sound (SOUNDLISTFILE, "crash");
@@ -343,7 +395,9 @@ static void shapegame_destroy_all_items()
     {
       /* Deleting the root item automatically deletes children items */
       gtk_object_destroy (GTK_OBJECT(shape_list_root_item));
+      shape_list_root_item = NULL;
       gtk_object_destroy (GTK_OBJECT(shape_root_item));
+      shape_root_item = NULL;
 
       g_hash_table_destroy (shapelist_table);
       shapelist_table=NULL;
@@ -361,7 +415,6 @@ static GnomeCanvasItem *shapegame_init_canvas(GnomeCanvasGroup *parent)
 			   "x", (double)gcomprisBoard->width/SHAPE_BOX_WIDTH_RATIO,
 			   "y", (double)0,
 			   NULL);
-  item_list = g_list_append (item_list, shape_list_root_item);
 
   shape_root_item = \
     gnome_canvas_item_new (parent,
@@ -369,7 +422,6 @@ static GnomeCanvasItem *shapegame_init_canvas(GnomeCanvasGroup *parent)
 			   "x", (double)gcomprisBoard->width/SHAPE_BOX_WIDTH_RATIO,
 			   "y", (double)0,
 			   NULL);
-  item_list = g_list_append (item_list, shape_root_item);
 
   return (item);
 }
@@ -434,7 +486,7 @@ static void
 add_shape_to_list_of_shapes(Shape *shape)
 {
   GnomeCanvasItem *item;
-  gint MAX_NUMBER_OF_SHAPES = 8;
+  gint MAX_NUMBER_OF_SHAPES = 10;
   double SHAPE_ELEM_NUMBER_IN_HEIGHT = (double)gcomprisBoard->height/MAX_NUMBER_OF_SHAPES;
 
   if(!shapelist_table)
@@ -458,7 +510,7 @@ add_shape_to_list_of_shapes(Shape *shape)
 	  if(pixmap)
 	    {
 	      double w, h;
-	      guint SHAPE_SIZE = SHAPE_ELEM_NUMBER_IN_HEIGHT - 20;
+	      guint SHAPE_SIZE = SHAPE_ELEM_NUMBER_IN_HEIGHT - 5;
 	      Shape *icon_shape;
 	      
 	      /* Calc a zoom factor so that the shape will fit in the shapelist
@@ -492,7 +544,6 @@ add_shape_to_list_of_shapes(Shape *shape)
 	      icon_shape->item = item;
 	      
 	      setup_item(item, icon_shape);
-	      item_list = g_list_append (item_list, item);
 	    }
 	}
     }
@@ -505,7 +556,7 @@ add_shape_to_list_of_shapes(Shape *shape)
 static Shape *find_closest_shape(double x, double y, double limit)
 {
   GList *list;
-  double goodDist = limit;
+  double goodDist = powf(limit,2);
   Shape *candidateShape = NULL;
 
   /* loop through all our shapes */
@@ -516,7 +567,7 @@ static Shape *find_closest_shape(double x, double y, double limit)
     if(shape->type==SHAPE_TARGET)
       {
 	/* Calc the distance between this shape and the given coord */
-	dist = sqrtf(powf((shape->x-x),2)*powf((shape->y-y),2));
+	dist = powf((shape->x-x),2) + powf((shape->y-y),2);
 	
 	if(dist<goodDist)
 	  {
@@ -554,6 +605,7 @@ static void shape_goes_back_to_list(Shape *shape, GnomeCanvasItem *item)
 			     shape->icon_shape->x - shape->x,
 			     shape->icon_shape->y - shape->y);
       gnome_canvas_item_show(shape->icon_shape->item);
+      gcompris_set_image_focus(shape->icon_shape->item, TRUE);
       shape->icon_shape=NULL;
 
       gnome_canvas_item_hide(item);
@@ -561,49 +613,6 @@ static void shape_goes_back_to_list(Shape *shape, GnomeCanvasItem *item)
     }
 }
 
-static void display_color_selector(GnomeCanvasGroup *parent)
-{
-  GdkPixbuf *pixmap;
-  GnomeCanvasItem *item = NULL;
-  gint x  = 0;
-  gint y  = 0;
-  gint x1 = 0;
-  gint c  = 0;
-
-  pixmap = gcompris_load_pixmap("draw/color-selector.jpg");
-  if(pixmap)
-    {
-      x = (gcomprisBoard->width
-	   - gcomprisBoard->width/SHAPE_BOX_WIDTH_RATIO*2
-	   - gdk_pixbuf_get_width(pixmap))/2 
-	+ gcomprisBoard->width/SHAPE_BOX_WIDTH_RATIO;
-      
-      y = gcomprisBoard->height - gdk_pixbuf_get_height(pixmap) - 5;
-
-      item = gnome_canvas_item_new (parent,
-				    gnome_canvas_pixbuf_get_type (),
-				    "pixbuf", pixmap, 
-				    "x", (double) x,
-				    "y", (double) y,
-				    NULL);
-      gdk_pixbuf_unref(pixmap);
-    }
-
-  
-  for(x1=x+26; x1<(x+26)+55*10; x1+=55)
-    {
-      item = gnome_canvas_item_new (parent,
-			     gnome_canvas_rect_get_type (),
-			     "x1", (double) x1,
-			     "y1", (double) y + 8,
-			     "x2", (double) x1 + 50,
-			     "y2", (double) y + gdk_pixbuf_get_height(pixmap) - 8,
-			     "fill_color", colorlist[c],
-			     NULL);
-      c++;
-    }
-}
-
 static gint
 item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 {
@@ -641,15 +650,28 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 	     {
 	       x = item_x;
 	       y = item_y;
+
+	       item_x = shape->x;
+	       item_y = shape->y;
 	       
 	       switch(shape->type)
 		 {
 		 case SHAPE_TARGET:
 		   gnome_canvas_item_hide(item);
+		   gcompris_set_image_focus(item, FALSE);
+
 		   if( shape->icon_shape!=NULL)
 		     {
 		       item = shape->icon_shape->item;
-		       gnome_canvas_item_show(item);
+		       item_x = x - (x - shape->x) * shape->icon_shape->w /
+			  shape->w;
+		       item_y = y - (y - shape->y) * shape->icon_shape->h /
+			  shape->h;
+		       gnome_canvas_item_move( item,
+					       item_x - shape->x,
+					       item_y - shape->y );
+		       gnome_canvas_item_show( item );
+		       gcompris_set_image_focus(item, TRUE);
 		       shape->icon_shape=NULL;
 		     }
 		   break;
@@ -657,8 +679,8 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 		   break;
 		 }
 	       /* This records the offset between the mouse pointer and the grabbed item center */
-	       offset_x = x - shape->x;
-	       offset_y = y - shape->y;
+	       offset_x = x - item_x;
+	       offset_y = y - item_y;
 	       
 	       if(item==NULL)
 		 return FALSE;
@@ -666,7 +688,7 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 	       fleur = gdk_cursor_new(GDK_FLEUR);
 
 	       /* In order to have our item above the others, I need to reparent it */
-	       gnome_canvas_item_reparent (item, shape_root_item);
+	       gnome_canvas_item_reparent (item, (GnomeCanvasGroup *)shape_root_item);
 	       gnome_canvas_item_raise_to_top(item);
 
 	       gnome_canvas_item_grab(item,
@@ -711,9 +733,9 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 	   gnome_canvas_item_ungrab(item, event->button.time);
 	   dragging = FALSE;
 
-	   gnome_canvas_item_reparent (item, shape_list_root_item);
+	   gnome_canvas_item_reparent (item, (GnomeCanvasGroup *)shape_list_root_item);
 
-	   targetshape = find_closest_shape(item_x, item_y, 1000);
+	   targetshape = find_closest_shape(item_x, item_y, 100);
 	   if(targetshape!=NULL)
 	     {
 	       /* Finish the placement of the grabbed item anyway */
@@ -765,7 +787,6 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 		   gdk_pixbuf_unref(pixmap);
 
 		   targetshape->bad_item = item;
-		   item_list = g_list_append (item_list, item);
 		   setup_item(item, targetshape);
 		 }
 
@@ -809,6 +830,103 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
    return FALSE;
  }
 
+/*
+ * Special mode to edit the board by moving the target point
+ */
+static gint
+item_event_edition(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
+{
+   static double x, y;
+   double new_x, new_y;
+   GdkCursor *fleur;
+   static int dragging;
+   double item_x, item_y;
+
+  if(!gcomprisBoard)
+    return FALSE;
+
+   if(shape==NULL) {
+     g_warning("Shape is NULL : Should not happen");
+     return FALSE;
+   }
+
+   item_x = event->button.x;
+   item_y = event->button.y;
+   gnome_canvas_item_w2i(item->parent, &item_x, &item_y);
+
+   switch (event->type) 
+     {
+     case GDK_BUTTON_PRESS:
+       switch(event->button.button) 
+         {
+         case 1:
+           if (event->button.state & GDK_SHIFT_MASK)
+             {
+	       /* Cheat code to save an XML file */
+	       write_xml_file("/tmp/gcompris-board.xml");
+	     }
+	   else
+	     {
+	       x = item_x;
+	       y = item_y;
+	       
+	       item = shape->target_point;
+
+	       fleur = gdk_cursor_new(GDK_FLEUR);
+
+	       gnome_canvas_item_grab(item,
+				      GDK_POINTER_MOTION_MASK | 
+				      GDK_BUTTON_RELEASE_MASK,
+				      fleur,
+				      event->button.time);
+	       gdk_cursor_destroy(fleur);
+	       dragging = TRUE;
+	     }
+	   break;
+	   
+         default:
+           break;
+         }
+       break;
+
+     case GDK_MOTION_NOTIFY:
+       if (dragging && (event->motion.state & GDK_BUTTON1_MASK)) 
+         {
+	   new_x = item_x;
+	   new_y = item_y;
+	       
+	   gnome_canvas_item_move(item, new_x - x, new_y - y);
+	   gnome_canvas_item_move(shape->item, new_x - x, new_y - y);
+
+	   // Update the shape's coordinate
+	   shape->x = shape->x + new_x - x;
+	   shape->y = shape->y + new_y - y;
+
+	   x = new_x;
+	   y = new_y;
+         }
+       break;
+           
+     case GDK_BUTTON_RELEASE:
+       if(dragging) 
+	 {
+
+	   gnome_canvas_item_ungrab(item, event->button.time);
+	   gnome_canvas_item_raise_to_top(item);
+	   dragging = FALSE;
+
+	 }
+       break;
+
+     default:
+       break;
+     }
+         
+   return FALSE;
+
+}
+
+
 static void
 setup_item(GnomeCanvasItem *item, Shape *shape)
 {
@@ -842,22 +960,21 @@ add_shape_to_canvas(Shape *shape)
       if(strcmp(shape->targetfile, UNDEFINED)!=0)
 	{
 	  targetpixmap = gcompris_load_pixmap(shape->targetfile);
+	  shape->w = (double)gdk_pixbuf_get_width(targetpixmap) * shape->zoomx;
+	  shape->h = (double)gdk_pixbuf_get_height(targetpixmap) *shape->zoomy;
+	  
 	  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
 					gnome_canvas_pixbuf_get_type (),
 					"pixbuf", targetpixmap, 
-					"x", (double)shape->x - 
-					(gdk_pixbuf_get_width(targetpixmap) * shape->zoomx)/2,
-					"y", (double)shape->y -
-					(gdk_pixbuf_get_height(targetpixmap) * shape->zoomy)/2,
-					"width", (double) gdk_pixbuf_get_width(targetpixmap) 
-					* shape->zoomx,
-					"height", (double) gdk_pixbuf_get_height(targetpixmap) 
-					* shape->zoomy,
+					"x", shape->x - shape->w / 2,
+					"y", shape->y - shape->h / 2,
+					"width",  shape->w,
+					"height", shape->h,
 					"width_set", TRUE, 
 					"height_set", TRUE,
 					NULL);
 	  gdk_pixbuf_unref(targetpixmap);
-	} 
+	}
       else
 	{
 	  /* Display a point to highlight the target location of this shape */
@@ -871,9 +988,12 @@ add_shape_to_canvas(Shape *shape)
 					"outline_color", "black",
 					"width_pixels", 0,
 					NULL);
+	  shape->target_point = item;
+	  gtk_signal_connect(GTK_OBJECT(item), "event",
+			     (GtkSignalFunc) item_event_edition,
+			     shape);
 	}
       gnome_canvas_item_lower_to_bottom(item);
-      item_list = g_list_append (item_list, item);
     }
   
   if(shape->points!=NULL)
@@ -892,19 +1012,22 @@ add_shape_to_canvas(Shape *shape)
       printf("it's an image ? shape->pixmapfile=%s\n", shape->pixmapfile);
       if(strcmp(shape->pixmapfile, UNDEFINED)!=0)
 	{
-	  printf("it's an image \n");
+	  printf("  Yes it is an image \n");
 	  pixmap = gcompris_load_pixmap(shape->pixmapfile);
 	  if(pixmap)
-	    {	
+	    {
+	      shape->w = (double)gdk_pixbuf_get_width(pixmap) * shape->zoomx;
+	      shape->h = (double)gdk_pixbuf_get_height(pixmap) * shape->zoomy;
+	       
 	      /* Display the shape itself but hide it until the user puts the right shape on it */
 	      /* I have to do it this way for the positionning (lower/raise) complexity          */
 	      item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
 					    gnome_canvas_pixbuf_get_type (),
 					    "pixbuf", pixmap, 
-					    "x", (double)shape->x-(gdk_pixbuf_get_width(pixmap) * shape->zoomx)/2,
-					    "y", (double)shape->y-(gdk_pixbuf_get_height(pixmap) * shape->zoomy)/2,
-					    "width", (double) gdk_pixbuf_get_width(pixmap) * shape->zoomx,
-					    "height", (double) gdk_pixbuf_get_height(pixmap) * shape->zoomy,
+					    "x", shape->x - shape->w / 2,
+					    "y", shape->y - shape->h / 2,
+					    "width", shape->w,
+					    "height", shape->h,
 					    "width_set", TRUE, 
 					    "height_set", TRUE,
 					    NULL);
@@ -922,23 +1045,24 @@ add_shape_to_canvas(Shape *shape)
   if(shape->type==SHAPE_TARGET || shape->type==SHAPE_DUMMY_TARGET)
     {
       setup_item(item, shape);
-      item_list = g_list_append (item_list, item);
 
       gnome_canvas_item_hide(item);
       add_shape_to_list_of_shapes(shape);
     }
   else if(shape->type==SHAPE_BACKGROUND)
-    gnome_canvas_item_lower_to_bottom(item);
+    {
+      gnome_canvas_item_lower_to_bottom(item);
+    }
 
 }
 
-void create_title(char *name, double x, double y, char *justification)
+static void create_title(char *name, double x, double y, char *justification)
 {
   GnomeCanvasItem *item;
   GdkFont *gdk_font;
 
   /* Load a gdk font */
-  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-180-*-*-*-*-*-*");
+  gdk_font = gdk_font_load (FONT_BOARD_MEDIUM);
 
   item = \
     gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
@@ -992,7 +1116,7 @@ create_shape(ShapeType type, char *name, char *pixmapfile, GnomeCanvasPoints* po
 
 
 static void
-add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
+add_xml_shape_to_data(xmlDocPtr doc, xmlNodePtr xmlnode, GNode * child)
 {
   char *name, *cx, *cy, *cd, *czoomx, *czoomy, *cposition, *ctype, *justification;
   char *pixmapfile = NULL;
@@ -1004,6 +1128,7 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
   guint position;
   ShapeType type = SHAPE_TARGET;
   Shape *shape;
+  xmlNodePtr xmlnamenode;
 
   if(/* if the node has no name */
      !xmlnode->name ||
@@ -1016,9 +1141,6 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
      )
     return;
   
-  /* get the name of the shape */
-  name = xmlGetProp(xmlnode,"name");
-
   pixmapfile = xmlGetProp(xmlnode,"pixmapfile");
   /* if unspecified, make it UNDEFINED */
   if(!pixmapfile) pixmapfile = UNDEFINED;
@@ -1097,6 +1219,27 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
   justification = xmlGetProp(xmlnode,"justification");
   if(!justification) justification = "GTK_JUSTIFICATION_CENTER"; /* GTK_JUSTIFICATION_CENTER is default */
 
+  /* get the name of the shape */
+  name = NULL;
+
+  xmlnamenode = xmlnode->xmlChildrenNode;
+  while (xmlnamenode != NULL) {
+    gchar *lang = xmlGetProp(xmlnamenode,"lang");
+    /* get the name of the shape */
+    if (!strcmp(xmlnamenode->name, "name")
+	&& (lang==NULL
+	    || !strcmp(lang, gcompris_get_locale())
+	    || !strncmp(lang, gcompris_get_locale(), 2)))
+      {
+	name = xmlNodeListGetString(doc, xmlnamenode->xmlChildrenNode, 1);
+	name = convertUTF8Toisolat1(name);
+      }
+    xmlnamenode = xmlnamenode->next;
+  }
+
+  /* If name is not given as an element, try to get it as a property */
+  if(!name)
+    name = xmlGetProp(xmlnode,"name");
 
   if(g_strcasecmp(xmlnode->name,"Shape")==0)
     {
@@ -1111,10 +1254,6 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
     {
       create_title(name, x, y, justification);
     }
-  else if (g_strcasecmp(xmlnode->name,"Option")==0)
-    {
-      display_color_selector(GNOME_CANVAS_GROUP(shape_root_item));
-    }
   
 }
 
@@ -1129,7 +1268,7 @@ parse_doc(xmlDocPtr doc)
   for(node = doc->children->children; node != NULL; node = node->next) {
     /* add the shape to the list, there are no children so
        we pass NULL as the node of the child */
-    add_xml_shape_to_data(node,NULL);
+    add_xml_shape_to_data(doc, node, NULL);
   }
   shuffle_shape_list();
 }
