@@ -25,15 +25,21 @@
 #include <gdk-pixbuf/gnome-canvas-pixbuf.h>
 
 #include "gcompris.h"
+#include "soundutil.h"
+
+#include "sys/wait.h"
+
+// returns a random integer in range [A,B]
+#define RAND(A,B) (A + (int)((float)(B-A+1)*rand()/(RAND_MAX+1.0)))
+
 
 GdkPixbuf	*gcompris_load_operation_pixmap(char operation);
 GdkPixbuf	*gcompris_load_number_pixmap(char number);
 GdkPixbuf	*gcompris_load_pixmap(char *pixmapfile);
 void		 gcompris_set_image_focus(GnomeCanvasItem *item, gboolean focus);
 gint		 gcompris_item_event_focus(GnomeCanvasItem *item, GdkEvent *event, void *unused);
-void		 gcompris_play_sound (const char *soundlistfile, const char *which);
-void 		gcompris_play_ogg(char *, ...);
 GcomprisBoard	*gcompris_read_xml_file(char *fname);
-gchar * convertUTF8Toisolat1(gchar * text);
+gchar		*convertUTF8Toisolat1(gchar * text);
+void item_absolute_move(GnomeCanvasItem *item, int x, int y);
 
 #endif
