@@ -1,6 +1,6 @@
 /* gcompris - gcompris.c
  *
- * Time-stamp: <2001/12/09 02:01:27 bruno>
+ * Time-stamp: <2002/04/13 23:54:29 bruno>
  *
  * Copyright (C) 2000,2001 Bruno Coudoin
  *
@@ -19,9 +19,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <signal.h>
+
 #include "gcompris.h"
 #include <popt-gnome.h>
 
+#include "cursor.h"
+
 GtkWidget *window;
 GtkWidget *drawing_area;
 GnomeCanvas *canvas;
@@ -30,14 +34,10 @@ GnomeCanvas *canvas_bg;
 
 //static gint pause_board_cb (GtkWidget *widget, gpointer data);
 static void quit_cb (GtkWidget *widget, gpointer data);
-static void about_cb (GtkWidget *widget, gpointer data);
-static void help_cb (GtkWidget *widget, gpointer data);
-static gint end_board_box (void);
 static gint board_widget_key_press_callback (GtkWidget   *widget,
 					    GdkEventKey *event,
 					    gpointer     client_data);
 
-
 GcomprisProperties	*properties = NULL;
 GcomprisBoard		*gcomprisBoardMenu = NULL;
 
@@ -55,6 +55,7 @@ static gchar           *gcompris_locale = NULL;
 #define P_SOUND		103
 #define P_MUTE		104
 #define P_VERSION	105
+#define P_CURSOR	106
 
 struct poptOption command_line[] = {
   {"fullscreen", 'f', POPT_ARGFLAG_ONEDASH, NULL, P_FULLSCREEN,
@@ -65,6 +66,8 @@ struct poptOption command_line[] = {
    N_("run gcompris with sound enabled.")},
   {"mute", 'm', POPT_ARGFLAG_ONEDASH, NULL, P_MUTE,
    N_("run gcompris without sound.")},
+  {"cursor", 'c', POPT_ARGFLAG_ONEDASH, NULL, P_CURSOR,
+   N_("run gcompris with the default gnome cursor.")},
   {"version", 'v', POPT_ARGFLAG_ONEDASH, NULL, P_VERSION,
    N_("Prints the version of " PACKAGE)},
   POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
@@ -113,6 +116,12 @@ GnomeCanvas *gcompris_get_canvas()
   return canvas;
 }
 
+GtkWidget *gcompris_get_window()
+{
+  return window;
+}
+
+
 GnomeCanvasItem *gcompris_set_background(GnomeCanvasGroup *parent, gchar *file)
 {
   GdkPixbuf *background_pixmap = NULL;
@@ -144,7 +153,7 @@ static void init_background()
 
   yratio=gdk_screen_height()/(float)(BOARDHEIGHT+BARHEIGHT);
   xratio=gdk_screen_width()/(float)BOARDWIDTH;
-    printf("The gdk_screen_width()=%f gdk_screen_height()=%f\n", 
+    printf("The gdk_screen_width()=%f gdk_screen_height()=%f\n",
   	 (double)gdk_screen_width(), (double)gdk_screen_height());
     printf("The xratio=%f yratio=%f\n", xratio, yratio);
 
@@ -158,6 +167,9 @@ static void init_background()
   /* Background area if ratio above 1 */
   if(xratio>=1.0 && properties->fullscreen)
     {
+      /* First, Remove the gnome crash dialog because it locks the user when in full screen */
+      signal(SIGSEGV, SIG_DFL);
+
       gnome_canvas_set_scroll_region (canvas_bg,
 				      0, 0,
 				      gdk_screen_width(),
@@ -220,9 +232,68 @@ static void init_background()
 
 }
 
+void gcompris_set_cursor(guint gdk_cursor_type)
+{
+  GdkCursor *cursor;
+
+  // Little hack to force gcompris to use the default gnome cursor
+  if(gdk_cursor_type==GCOMPRIS_DEFAULT_CURSOR)
+    gdk_cursor_type=properties->defaultcursor;
+
+  // I suppose there is less than GCOMPRIS_FIRST_CUSTOM_CURSOR cursors defined in gdkcursors.h !
+  if (gdk_cursor_type < GCOMPRIS_FIRST_CUSTOM_CURSOR) {
+    cursor = gdk_cursor_new(gdk_cursor_type);
+    gdk_window_set_cursor	 (window->window, cursor);
+    gdk_cursor_destroy(cursor);
+  } else { // we use a custom cursor
+    GdkColor fg, bg;
+    //    static const gchar * cursor;
+    static const gchar ** bits;
+    
+    switch (gdk_cursor_type) {
+    case GCOMPRIS_BIG_RED_ARROW_CURSOR : 
+      bits = big_red_arrow_cursor_bits; 
+      break;
+    case GCOMPRIS_BIRD_CURSOR : 
+      bits = bird_cursor_bits; 
+      break;
+    case GCOMPRIS_LINE_CURSOR : 
+      bits = big_red_line_cursor_bits; 
+      break;
+    case GCOMPRIS_RECT_CURSOR : 
+      bits = big_red_rectangle_cursor_bits; 
+      break;
+    case GCOMPRIS_FILLRECT_CURSOR : 
+      bits = big_red_filledrectangle_cursor_bits; 
+      break;
+    case GCOMPRIS_CIRCLE_CURSOR : 
+      bits = big_red_circle_cursor_bits; 
+      break;
+    case GCOMPRIS_FILLCIRCLE_CURSOR : 
+      bits = big_red_filledcircle_cursor_bits; 
+      break;
+    case GCOMPRIS_FILL_CURSOR : 
+      bits = big_red_fill_cursor_bits; 
+      break;
+    case GCOMPRIS_DEL_CURSOR : 
+      bits = big_red_del_cursor_bits; 
+      break;
+    case GCOMPRIS_SELECT_CURSOR : 
+      bits = big_red_select_cursor_bits; 
+      break;
+    default : bits = big_red_arrow_cursor_bits;
+    }
+    
+    gdk_color_parse("rgb:FFFF/FFFF/FFFF",&fg);
+    gdk_color_parse("rgb:FFFF/3FFF/0000",&bg);
+    cursor = gdk_cursor_new_from_data(bits, 40 , 40, &fg, &bg, 0, 0);
+    gdk_window_set_cursor(window->window, cursor);
+    gdk_cursor_destroy(cursor);
+  }
+}
+
 static void setup_window ()
 {
-
   window = gnome_app_new (PACKAGE, _("GCompris I Have Understood"));
 
   gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
@@ -240,12 +311,8 @@ static void setup_window ()
     gnome_win_hints_set_layer (GTK_WIDGET (window),  WIN_LAYER_ABOVE_DOCK);
   //  gnome_win_hints_set_state(GTK_WIDGET (window),  WIN_STATE_FIXED_POSITION);
 
-
-  /*  GdkCursor *hand_cursor; */
-  /*  hand_cursor = gdk_cursor_new(GDK_HAND2);
-      gdk_window_set_cursor	 (window->window, hand_cursor);
-      gdk_cursor_destroy(hand_cursor);
-  */
+  // Set the cursor
+  gcompris_set_cursor(GCOMPRIS_DEFAULT_CURSOR);
 
   /* -------------- */
 
@@ -268,6 +335,7 @@ static void setup_window ()
   /*
   canvas     = GNOME_CANVAS(gnome_canvas_new_aa ());
   canvas_bar = GNOME_CANVAS(gnome_canvas_new_aa ());
+  canvas_bg = GNOME_CANVAS(gnome_canvas_new_aa ());
   */
 
 
@@ -283,7 +351,7 @@ static void setup_window ()
     {
       gdk_window_set_decorations (window->window, 0);
       gdk_window_set_functions (window->window, 0);
-      gtk_widget_set_uposition (window, 0, 0);  
+      gtk_widget_set_uposition (window, 0, 0);
     }
 
   init_plugins();
@@ -298,105 +366,9 @@ static void setup_window ()
 
   board_play (gcomprisBoardMenu);
 
-  /* Menu creation */  
-
-  //  gnome_app_create_menus (GNOME_APP (window), main_menu);
-  
-  //  gnome_app_install_menu_hints (GNOME_APP (window), main_menu);
-
   init_background();
 }
 
-/* It implements gcompris's own way to load help in order to be
- * Package relocatable.
- * Unfortunatly, this does not supports I18N
- */
-static void help_cb (GtkWidget *widget, gpointer data)
-{
-  gnome_help_goto(NULL, PACKAGE_HELP_DIR "/C/gcompris.html");
-}
-
-static void about_cb (GtkWidget *widget, gpointer data)
-{
-  static GtkWidget *about;
-  GtkWidget *l, *hbox;
-
-
-  const gchar *authors[] = {
-    "Bruno Coudoin <bruno.coudoin@free.fr>",
-    NULL
-  };
-
-  if (about != NULL) {
-    gdk_window_raise (about->window);
-    gdk_window_show (about->window);
-    return;
-  }
-
-  about = gnome_about_new ("GCompris", VERSION,
-                        ("Copyright 2000,2001 Bruno Coudoin (Released under the GPL)"),
-                        authors,
-                        _("A simple educational board based game for children starting at 3. This software is a GNU Package"),
-                        PACKAGE_DATA_DIR "/gcompris/gcompris-about.jpg");
-
-  hbox = gtk_hbox_new (TRUE, 0);
-  l = gnome_href_new ("http://www.ofset.org/gcompris",
-		      _ ("GCompris Home Page"));
-  gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
-  l = gnome_href_new ("http://www.ofset.org",
-		      _ ("An OFSET development"));
-  gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
-		      hbox, TRUE, FALSE, 0);
-  gtk_widget_show_all (hbox);
-
-  gtk_object_set_data (GTK_OBJECT (about), "about", about);
-  gtk_window_set_modal (GTK_WINDOW (about), TRUE);
-
-  gtk_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
-		      (gtk_widget_destroyed), &about);
-  gnome_dialog_set_parent (GNOME_DIALOG (about), GTK_WINDOW (window));
-  
-  gtk_widget_show (about);
-}
-
-/*
- * Confirmation dialog
- */
-static gint end_board_box ()
-{
-  static GtkWidget *box;
-  gint status;
-
-  if (box)
-    return 0;
-
-  board_pause();
-
-  box = gnome_message_box_new (
-			       _("Do you really want to quit GCompris?"),
-			       GNOME_MESSAGE_BOX_QUESTION,
-			       GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
-			       NULL);
-  gnome_dialog_set_parent (GNOME_DIALOG (box), GTK_WINDOW
-			   (window));
-  gnome_dialog_set_default (GNOME_DIALOG (box), 0);
-  status = gnome_dialog_run (GNOME_DIALOG (box));
-  box = NULL;
-  
-  board_pause();
-
-  return (status);
-}
-
-/*
-static gint pause_board_cb (GtkWidget *widget, gpointer data)
-{
-  board_pause();
-  return(TRUE);
-}
-*/
-
 void gcompris_end_board()
 {
   if (get_current_gcompris_board()->previous_board == NULL)
@@ -437,20 +409,20 @@ static void load_properties ()
  */
 gchar *gcompris_get_locale()
 {
+  char *locale;
 
-  /* First check gcompris got overrided by the user */
-  if(gcompris_locale !=NULL)
+  /* First check locale got overrided by the user */
+  if(gcompris_locale != NULL)
     return(gcompris_locale);
 
-  if(!strcmp(setlocale(0, NULL), "C"))
-    {
-      /* NO Locale, Try to use the English Locale */
-      return("en");
-    } 
-  else 
-    {
-      return(setlocale(0, NULL));
-    }
+  locale = getenv("LC_ALL");
+  if(locale == NULL)
+    locale = getenv("LANG");
+
+  if(locale!=NULL)
+    return(locale);
+
+  return("en");
 }
 
 /*
@@ -460,9 +432,9 @@ gchar *gcompris_get_locale()
 void gcompris_set_locale(gchar *locale)
 {
 
-  gcompris_locale = setlocale(LC_ALL, locale);
+  gcompris_locale = g_strdup(setlocale(LC_ALL, locale));
   printf("gcompris_set_locale requested %s got %s\n", locale, gcompris_locale);
-  /* FIXME: This does not update gettext translation */
+  /* WARNING: This does not update gettext translation */
 }
 
 /*****************************************
@@ -476,17 +448,26 @@ main (int argc, char *argv[])
   int c;
   poptContext optCon;
 
+  /* To have some real random behaviour */
   srand (time (NULL));
 
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
 
-  gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, command_line, 0, &optCon);
-
-  optCon = poptGetContext (NULL, argc, argv, command_line, 0);
-
   load_properties ();
 
+  // Set the default gcompris cursor
+  properties->defaultcursor = GCOMPRIS_DEFAULT_CURSOR;
+
+  // Set the user's choice locale
+  gcompris_set_locale(properties->locale);
+  
+  initSound();
+  
+  gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, command_line, 0, &optCon);
+  
+  optCon = poptGetContext (NULL, argc, argv, command_line, 0);
+
   /*------------------------------------------------------------*/
   while ((c = poptGetNextOpt (optCon)) != -1)
     {
@@ -498,7 +479,7 @@ main (int argc, char *argv[])
 		  VERSION);
 	  exit (0);
 	  break;
-	  
+
 	case P_FULLSCREEN:
 	  properties->fullscreen = TRUE;
 	  break;
@@ -518,6 +499,11 @@ main (int argc, char *argv[])
 	  properties->music = TRUE;
 	  properties->fx = TRUE;
 	  break;
+
+	case P_CURSOR:
+	  g_warning("Default gnome cursor enabled");
+	  properties->defaultcursor = GDK_LEFT_PTR;
+	  break;
 	}
     }
 
@@ -542,11 +528,18 @@ main (int argc, char *argv[])
 
   gtk_widget_show (window);
 
+  if (properties->music)
+    {
+      gcompris_play_ogg("intro", "welcome", NULL);
+    }
+  else
+    gcompris_play_ogg("welcome", NULL);
+
   gtk_main ();
   return 0;
 }
 
-
+
 /* Local Variables: */
 /* mode:c */
 /* eval:(load-library "time-stamp") */
